/*
**  adc.c - Sample code for HyperpanelOS ==================================  **
**                                                                           **
**  This simple code is located into the Application Container, it is run    **
**  by the VMK sub-operating system. On the other hand, the I/O container    **
**  runs all the drivers that are VMIO Finite State machines.                **
**                                                                           **
**  The goal of this small aplication is to use internal ADC of the MCU.     **
**                                                                           **
**  =======================================================================  **
*/

/* Purpose of this module ---------------------------------------------------
   
   The STM32F407 include a 12-bit ADC. It include 19 multiplexed channels,
   allowing to measure signals from 16 external sources, two internal sources
   and a rereference voltage channel. The A/D conversion can be performed in
   single or continuous conversion mode. The result of ADC is stored into a
   16-bit register.

   To access to HyperpanelOS ADC driver, we use the "iolib" simplified
   interface to "drv.c" driver. So this tutorial also serves as a "how to use"
   of the "iolib" library.

   In this simple example, we use the ADC in single mode to get a variable
   voltage from a potentiometer (from 0mv to 3000mV) by reading the sub-channel
   number 1 from the ADC1 device.

   We also get values from the temperature sensor of the MCU by reading the
   sub-channel 16 from the ADC1 device.

   The boards contains ADC driver static configuration :

   static const Adc_inpdef adc_inpdef[ADC_NBINP] = /+ High-level device def  +/
     BEGIN                             /+ ---------------------------------- +/
       "ADC1"                        , /+ uname     MUX input name           +/
        0                            , /+ fastcap   Fast sampling capacity   +/
        0                            , /+ gaincap   Amplifier available      +/
        12                           , /+ swpos     MUX input selector (PC2) +/
        0                            , /+ lsfifonum FIFO low-level identifier+/
        0                            , /+ lsfdmanum DMA low-level identifier +/
   
       "TEMP"                        , /+ uname     MUX input name           +/
        0                            , /+ fastcap   Fast sampling capacity   +/
        0                            , /+ gaincap   Amplifier available      +/
        16                           , /+ swpos     MUX input selector       +/
        0                            , /+ lsfifonum FIFO low-level identifier+/
        0                            , /+ lsfdmanum DMA low-level identifier +/
     END                             ; /+ ---------------------------------- +/

    Potentiometer voltage : The potentiometer is powered by the board (GND and
3V pins), the analog signal from the potentiometer deliver a continus tension
from 0V to 3v, and is connected to PC2 (cf. board pins definitions).

   Temperature : The temperature sensor has to generate a voltage that varies
linearly with temperature. It is internally connected to the ADC1_IN16 input
channel which is used to convert the sensor output voltage into a digital value.
The conversion is (for the stm32m4, cf. RM0090 p 413 "Reading the tempoerature"):

    Temperature (in C) = ((Vsense - V25) / Avg_Slope) + 25

with (cf. stm32F407 datasheet p 138):

    Vsense = Value read from the ADC data register
    V25       = 0.76 V
    Avg_Slope = 2.5 mV/C

*/

/* Includes files and external references ...................................*/

#include <hypos.h>                     // Hyperpanel OS basic interfaces
#include <drv_asy.h>                   // Prototype of "asy_write"
#include <tsk_iolib.h>                 // Prototype of "io_open"


/* Internal defines of this module ------------------------------------------*/

#define  TICK                   10000  // Code for tick event
                                
                                
/* Internal global variables of this module ---------------------------------*/

static unsigned int   idto           ; // Timer identifier
static int            adc_iod        ; // IOD of device ADC
static int            adc_iod1       ; // IOD of device ADC\ADC1
static int            adc_iod2       ; // IOD of device ADC\TEMP
static int            cmess = 0      ; // Counter of messages
static char           mess[80]       ; // Message for traces 


/*  List of tags for the "open_xyz" procedures ------------------------------*/

static const char *adc_taglist      = ""                                      ; 


/*  Prototypes --------------------------------------------------------------*/

static int  wait_evt     (void)                                               ;
static void open_adc     (void)                                               ;
static void trace        (char*)                                              ;
    

/* Beginning of the code ----------------------------------------------------

loop_app_tsk        Entry point for create_task - Main event loop
wait_evt            Wait for events
open_adc            Open the ADC driver
trace               Send a trace on serial port

*/


/*  Procedure loop_app_tsk ----------------------------------------------------

    Purpose : This is our task main loop.
*/

int loop_app_tsk (void *param)
  {
    int             ev               ; // Our event
    char            list[80]         ; // Drive / tag list
    int             nb               ; // Number of drivers
    char            name[20]         ; // Driver name
    char            valstr[20]       ; // String value from tag
    int             valint           ; // Integer value from tag
    float           valfloat         ; // Integer value from tag
    char           *l, *p            ; // Pointer on "list"
    int             i                ; // Driver counter
    int             j = 0            ; // Character counter
    int             ret              ; // Return procedure code
    char            e[4],f[4]        ; // Integer / float part of temperature
    char            str[20]          ; // Character string

    trace("Hyperpanel OS - Tutorial adc app") ; // Message

/*****************************************************************************
 * Step 1 : Using information procedures to acces global info of all         *
 *          the drivers of the board.                                        *
 *****************************************************************************/

    ret = io_list_drivers(list       , // Ge the list of all the available
                   sizeof(list),&nb) ; // character drivers

    hsprintf(mess,"(list_drv) ret=%d nb=%d",ret,nb); // Return value
    trace(mess)                                    ; // of procedure

    for (l=list,p=l,i=0; i LT nb; i++) // Loop on all the driver name
      {
        while(*p NE '\n' LOGAND        // Search for the end of the line or
              *p NE 0)                 // the end of the list.
         {
           p++                    ;    // Next character
           j++                    ;    // Update character counter
         }                        

        Strncpy(name,l,j)      ;    // End of line reached, get the
        name[j] = 0            ;    // driver name and close the string

        trace(name)            ;    // Trace the current driver name
        j = 0                     ;    // Reset character counter
        p++                       ;    // Point on next character of the list
        l = p                     ;    // Point of first character of next name

      }

/*****************************************************************************
 * Step 1 : Driver initialization.                                           *
 *****************************************************************************/

  open_adc()                         ; // Open the ADC driver 

/*****************************************************************************
 * Step 1 : We start a timer that will send us an event every 1 second so    *
 * ------   that we can update the time.                                     *
 *****************************************************************************/

    set_uto(CLOCK      ,               // Timer mode: clock
            1000       ,               // Duration in milliseconds
            TICK       ,               // Event code
            0          ,               // Event reserve field
            &idto        )           ; // Timer identifier


/*****************************************************************************
 * Step 2 : Here is our main event loop. For each loop, we do as follows :   *
 * -------  - First we wait for an event.                                    *
 *          - Then if the event code is TICK we print "Hello World"          *
 *****************************************************************************/

    wait_evt                         : // Beginning of loop label

    ev = wait_evt()                  ; // Unschedule until an event is
                                       // received

    if ( ev == TICK )                  // If the event is the tick event
      {                                //
        trace("TICK event")         ;  // Send a trace

                                       // Voltage from potentiometer.

        strcpy(list,"SAMPLE")        ; // Get value from ADC/ADC1
        ret = io_getval(adc_iod1     , //
                        list         , //
                        sizeof(list)); //

        Strcpy(valstr,&list[7])      ; // Tag to int conversion.
        hsscanf(valstr,"%x",&valint) ; // 
        valint = (valint*3000)/4095  ; // Value normalization to mV.

        hsprintf(mess                , // Format the message with 
                 "Potentiometer %dmV", // current voltage in mV.
                 valint             ); //
        trace(mess)                  ; // Send a trace.

                                       // Temperature

        strcpy(list,"SAMPLE")        ; // Get value from ADC/TEMP
        ret = io_getval(adc_iod2     , //
                        list         , //
                        sizeof(list)); //

        Strcpy(valstr,&list[7])      ; // Tag to int conversion.
        hsscanf(valstr,"%x",&valint) ; // 
                                     
                                       // Convert value to temperature
                                       // (cf. formula in explanations).

        valfloat = ((valint/1000.f - 0.76f)/2.5f) + 22.0f ;

        valint = valfloat*10         ; // Integer part for temperature.
        hsprintf(str,"%d",valint)    ;
        Strncpy(e,str,2)  ; e[2]=0   ;
        Strcpy (f,&str[2]); f[1]=0   ; // Fractional part of temperature.

        hsprintf(mess                , // Format the message with 
               "Temperature   %s.%sC", // current voltage in mV.
                 e,f                ); //
        trace(mess)                  ; // Send a trace.
      }
    else
      {
        trace("Other event")        ;  // Send a trace
      }

    goto wait_evt                    ; // Wait for the next event

    return 0                         ;
  }

/*  Procedure wait_evt ------------------------------------------------------*/
/*
    Purpose : Unschedule until the next event is received, whatever it is.
*/

static int wait_evt (void)
  {
    unsigned int waitlist[8][3]      ; // Parameter of "waitevt_task"

/*****************************************************************************
 * Step 1 : Build a list with one WAIT_CODEINT entry that will accept all    *
 * ------   the event codes ranging from 0 to 20000. Then call               *
 *          "waitevt_task", we will be unscheduled until the next event will *
 *          be received                                                      *
 *****************************************************************************/

    waitlist[0][0]  = WAIT_CODEINT   ; // All events with
    waitlist[0][1]  = 0              ; // a code between 0
    waitlist[0][2]  = 20000          ; // and 20000

    waitevt_task(waitlist ,            // Address of waiting list
                 1        ,            // Size of "waitlist[]"
                 0        ,            // maximum waiting time = no
                 0        )          ; // Do not purge previous events

/*****************************************************************************
 * Step 2 : Here we are scheduled again. The VMK has written into its        *
 * ------   global variable "task_evt" a copy of the event that has          *
 *          scheduled us again.                                              *
 *****************************************************************************/

    return task_evt.code             ; // Return event code
  }

/*  Procedure open_adc ------------------------------------------------------*/
/*
    Purpose : Open the ADC driver.
*/

static void open_adc(void)
  {
    int   ret                        ;  // Return procedure value

    char            list[200]        ; // Device list names for a driver
    char            name[20]         ; // Tag name
    char            taglist[80]      ; // Tag list
    int             nb               ; // Nomber of devices for a driver
    char           *l, *p            ; // Pointer on "list"
    int             i                ; // Tag counter
    int             j = 0            ; // Character counter

/* open .....................................................................*/

    ret = io_open("ADC","ADC1",adc_taglist,&adc_iod) ; // Open ADC\ADC1

    hsprintf(mess,"(open) adc_iod = 0x%08x ret=%d",
                                      adc_iod,ret);
    trace(mess)                                   ;

/* devices list .............................................................*/

    ret = io_list_devices("ADC",list,sizeof(list),&nb);

    hsprintf(mess,"(list_dev) ret=%d nb=%d", //
                                    ret,nb); // Return value
    trace(mess)                            ; // of procedure
    trace(list)                            ; // Device list.

/* Device tags list .........................................................*/

    ret = io_list_tags("ADC",list,sizeof(list),&nb);

    hsprintf(mess,"(list_tag) ret=%d nb=%d", //
                                    ret,nb); // Return value
    trace(mess)                            ; // of procedure

    for (l=list,p=l,i=0; i LT nb; i++)       // Loop on all the driver name
      {
        while(*p NE '\n' LOGAND              // Search for the end of line or
              *p NE 0)                       // the end of the list.
         {
           p++                             ; // Next character
           j++                             ; // Update character counter
         }                        

        Strncpy(name,l,j)                  ; // End of line reached, get the
        name[j] = 0                        ; // driver name and close the string

        j = 0                              ; // Reset character counter
        p++                                ; // Point on next char of the list
        l = p                              ; // Point of first char of next name

        if (!strcmp(name,"NUMDEV")) continue;// Just to avoid break point

        strcpy(taglist,name)               ; // Get default val of of the

        hsprintf(mess,"(tag) <%s>",          //
                                   taglist); //
        trace(mess)                        ; //

        io_getval(adc_iod,taglist          , // readable tags.
                   sizeof(taglist))        ; //
        hsprintf(mess,"(getval) <%s>",       //
                                   taglist); //
        trace(mess)                        ; //
      }

/* sub-channel allocation ...................................................*/

    ret = io_alloc_sub(adc_iod,"UNAME=ADC1",&adc_iod1);
    hsprintf(mess,"(alloc_sub) adc_iod1=0x%08x ret=%d",
                                         adc_iod1,ret);
    trace(mess)                                       ;

    ret = io_alloc_sub(adc_iod,"UNAME=TEMP",&adc_iod2);
    hsprintf(mess,"(alloc_sub) adc_iod2=0x%08x ret=%d",
                                         adc_iod2,ret);
    trace(mess)                                       ;

    strcpy(taglist,"FASTCAP")                         ; 
    ret = io_getval(adc_iod1,taglist,sizeof(taglist)) ;
    hsprintf(mess,"(getval) <%s> ret=%d",taglist,ret) ;
    trace(mess)                                       ;

    strcpy(taglist,"GAINCAP")                         ;  
    ret = io_getval(adc_iod1,taglist,sizeof(taglist)) ;
    hsprintf(mess,"(getval) <%s> ret=%d",taglist,ret) ;
    trace(mess)                                       ;

    strcpy(taglist,"MODE")                            ;  
    ret = io_getval(adc_iod1,taglist,sizeof(taglist)) ;
    hsprintf(mess,"(getval) <%s> ret=%d",taglist,ret) ;
    trace(mess)                                       ;

    strcpy(taglist,"IHYST")                           ;  
    ret = io_getval(adc_iod1,taglist,sizeof(taglist)) ;
    hsprintf(mess,"(getval) <%s> ret=%d",taglist,ret) ;
    trace(mess)                                       ;

    strcpy(taglist,"GAIN")                            ;  
    ret = io_getval(adc_iod1,taglist,sizeof(taglist)) ;
    hsprintf(mess,"(getval) <%s> ret=%d",taglist,ret) ;
    trace(mess)                                       ;

    strcpy(taglist,"SIGN")                            ;  
    ret = io_getval(adc_iod1,taglist,sizeof(taglist)) ;
    hsprintf(mess,"(getval) <%s> ret=%d",taglist,ret) ;
    trace(mess)                                       ;

    strcpy(taglist,"SCALING")                         ;  
    ret = io_getval(adc_iod1,taglist,sizeof(taglist)) ;
    hsprintf(mess,"(getval) <%s> ret=%d",taglist,ret) ;
    trace(mess)                                       ;

    strcpy(taglist,"SCAMUL")                          ;  
    ret = io_getval(adc_iod1,taglist,sizeof(taglist)) ;
    hsprintf(mess,"(getval) <%s> ret=%d",taglist,ret) ;
    trace(mess)                                       ;

    strcpy(taglist,"SCADIV")                          ;  
    ret = io_getval(adc_iod1,taglist,sizeof(taglist)) ;
    hsprintf(mess,"(getval) <%s> ret=%d",taglist,ret) ;
    trace(mess)                                       ;
  }

/*  Procedure trace ---------------------------------------------------------*/
/*
    Purpose : Send a trace on serial port.
*/

static void trace(char *s)
  {
    char str[255]                    ; // Message to be sent

    hsprintf(str                     , // We build in "str" the message
           "%6d  %s\r\n"             , // to be sent
            cmess ++ ,s            ) ; //

    asy_write(0                      , // We send on ASY0
            (unsigned char*)str      , // the "mess" message
            strlen(str)            ) ; // Count of bytes to be sent
  }
